/*
    +--------------------------------------------------------------------+
    | PECL :: http                                                       |
    +--------------------------------------------------------------------+
    | Redistribution and use in source and binary forms, with or without |
    | modification, are permitted provided that the conditions mentioned |
    | in the accompanying LICENSE file are met.                          |
    +--------------------------------------------------------------------+
    | Copyright (c) 2004-2014, Michael Wallner <mike@php.net>            |
    +--------------------------------------------------------------------+
*/

#ifndef PHP_HTTP_UTF8_H
#define PHP_HTTP_UTF8_H

typedef struct utf8_range {
	unsigned int start;
	unsigned int end;
	unsigned char step;
} utf8_range_t;

static const unsigned char utf8_mblen[256] = {
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
    2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
    4,4,4,4,4,4,4,4,5,5,5,5,6,6,6,6
};

static const unsigned char utf8_mask[] = {
		0, 0x7f, 0x1f, 0x0f, 0x07, 0x03, 0x01
};

static const utf8_range_t utf8_ranges[] = {
/* BEGIN::UTF8TABLE */
	{    0x0041,     0x005A, 1},
	{    0x0061,     0x007A, 1},
	{    0x00AA,          0, 0},
	{    0x00B5,          0, 0},
	{    0x00BA,          0, 0},
	{    0x00C0,     0x00D6, 1},
	{    0x00D8,     0x00F6, 1},
	{    0x00F8,     0x02C1, 1},
	{    0x02C6,     0x02D1, 1},
	{    0x02E0,     0x02E4, 1},
	{    0x02EC,          0, 0},
	{    0x02EE,          0, 0},
	{    0x0345,          0, 0},
	{    0x0370,     0x0374, 1},
	{    0x0376,     0x0377, 1},
	{    0x037A,     0x037D, 1},
	{    0x037F,          0, 0},
	{    0x0386,          0, 0},
	{    0x0388,     0x038A, 1},
	{    0x038C,          0, 0},
	{    0x038E,     0x03A1, 1},
	{    0x03A3,     0x03F5, 1},
	{    0x03F7,     0x0481, 1},
	{    0x048A,     0x052F, 1},
	{    0x0531,     0x0556, 1},
	{    0x0559,          0, 0},
	{    0x0561,     0x0587, 1},
	{    0x05B0,     0x05BD, 1},
	{    0x05BF,          0, 0},
	{    0x05C1,     0x05C2, 1},
	{    0x05C4,     0x05C5, 1},
	{    0x05C7,          0, 0},
	{    0x05D0,     0x05EA, 1},
	{    0x05F0,     0x05F2, 1},
	{    0x0610,     0x061A, 1},
	{    0x0620,     0x0657, 1},
	{    0x0659,     0x0669, 1},
	{    0x066E,     0x06D3, 1},
	{    0x06D5,     0x06DC, 1},
	{    0x06E1,     0x06E8, 1},
	{    0x06ED,     0x06FC, 1},
	{    0x06FF,          0, 0},
	{    0x0710,     0x073F, 1},
	{    0x074D,     0x07B1, 1},
	{    0x07C0,     0x07EA, 1},
	{    0x07F4,     0x07F5, 1},
	{    0x07FA,          0, 0},
	{    0x0800,     0x0817, 1},
	{    0x081A,     0x082C, 1},
	{    0x0840,     0x0858, 1},
	{    0x0860,     0x086A, 1},
	{    0x08A0,     0x08B4, 1},
	{    0x08B6,     0x08BD, 1},
	{    0x08D4,     0x08DF, 1},
	{    0x08E3,     0x08E9, 1},
	{    0x08F0,     0x093B, 1},
	{    0x093D,     0x094C, 1},
	{    0x094E,     0x0950, 1},
	{    0x0955,     0x0963, 1},
	{    0x0966,     0x096F, 1},
	{    0x0971,     0x0983, 1},
	{    0x0985,     0x098C, 1},
	{    0x098F,     0x0990, 1},
	{    0x0993,     0x09A8, 1},
	{    0x09AA,     0x09B0, 1},
	{    0x09B2,          0, 0},
	{    0x09B6,     0x09B9, 1},
	{    0x09BD,     0x09C4, 1},
	{    0x09C7,     0x09C8, 1},
	{    0x09CB,     0x09CC, 1},
	{    0x09CE,          0, 0},
	{    0x09D7,          0, 0},
	{    0x09DC,     0x09DD, 1},
	{    0x09DF,     0x09E3, 1},
	{    0x09E6,     0x09F1, 1},
	{    0x09FC,          0, 0},
	{    0x0A01,     0x0A03, 1},
	{    0x0A05,     0x0A0A, 1},
	{    0x0A0F,     0x0A10, 1},
	{    0x0A13,     0x0A28, 1},
	{    0x0A2A,     0x0A30, 1},
	{    0x0A32,     0x0A33, 1},
	{    0x0A35,     0x0A36, 1},
	{    0x0A38,     0x0A39, 1},
	{    0x0A3E,     0x0A42, 1},
	{    0x0A47,     0x0A48, 1},
	{    0x0A4B,     0x0A4C, 1},
	{    0x0A51,          0, 0},
	{    0x0A59,     0x0A5C, 1},
	{    0x0A5E,          0, 0},
	{    0x0A66,     0x0A75, 1},
	{    0x0A81,     0x0A83, 1},
	{    0x0A85,     0x0A8D, 1},
	{    0x0A8F,     0x0A91, 1},
	{    0x0A93,     0x0AA8, 1},
	{    0x0AAA,     0x0AB0, 1},
	{    0x0AB2,     0x0AB3, 1},
	{    0x0AB5,     0x0AB9, 1},
	{    0x0ABD,     0x0AC5, 1},
	{    0x0AC7,     0x0AC9, 1},
	{    0x0ACB,     0x0ACC, 1},
	{    0x0AD0,          0, 0},
	{    0x0AE0,     0x0AE3, 1},
	{    0x0AE6,     0x0AEF, 1},
	{    0x0AF9,     0x0AFC, 1},
	{    0x0B01,     0x0B03, 1},
	{    0x0B05,     0x0B0C, 1},
	{    0x0B0F,     0x0B10, 1},
	{    0x0B13,     0x0B28, 1},
	{    0x0B2A,     0x0B30, 1},
	{    0x0B32,     0x0B33, 1},
	{    0x0B35,     0x0B39, 1},
	{    0x0B3D,     0x0B44, 1},
	{    0x0B47,     0x0B48, 1},
	{    0x0B4B,     0x0B4C, 1},
	{    0x0B56,     0x0B57, 1},
	{    0x0B5C,     0x0B5D, 1},
	{    0x0B5F,     0x0B63, 1},
	{    0x0B66,     0x0B6F, 1},
	{    0x0B71,          0, 0},
	{    0x0B82,     0x0B83, 1},
	{    0x0B85,     0x0B8A, 1},
	{    0x0B8E,     0x0B90, 1},
	{    0x0B92,     0x0B95, 1},
	{    0x0B99,     0x0B9A, 1},
	{    0x0B9C,          0, 0},
	{    0x0B9E,     0x0B9F, 1},
	{    0x0BA3,     0x0BA4, 1},
	{    0x0BA8,     0x0BAA, 1},
	{    0x0BAE,     0x0BB9, 1},
	{    0x0BBE,     0x0BC2, 1},
	{    0x0BC6,     0x0BC8, 1},
	{    0x0BCA,     0x0BCC, 1},
	{    0x0BD0,          0, 0},
	{    0x0BD7,          0, 0},
	{    0x0BE6,     0x0BEF, 1},
	{    0x0C00,     0x0C03, 1},
	{    0x0C05,     0x0C0C, 1},
	{    0x0C0E,     0x0C10, 1},
	{    0x0C12,     0x0C28, 1},
	{    0x0C2A,     0x0C39, 1},
	{    0x0C3D,     0x0C44, 1},
	{    0x0C46,     0x0C48, 1},
	{    0x0C4A,     0x0C4C, 1},
	{    0x0C55,     0x0C56, 1},
	{    0x0C58,     0x0C5A, 1},
	{    0x0C60,     0x0C63, 1},
	{    0x0C66,     0x0C6F, 1},
	{    0x0C80,     0x0C83, 1},
	{    0x0C85,     0x0C8C, 1},
	{    0x0C8E,     0x0C90, 1},
	{    0x0C92,     0x0CA8, 1},
	{    0x0CAA,     0x0CB3, 1},
	{    0x0CB5,     0x0CB9, 1},
	{    0x0CBD,     0x0CC4, 1},
	{    0x0CC6,     0x0CC8, 1},
	{    0x0CCA,     0x0CCC, 1},
	{    0x0CD5,     0x0CD6, 1},
	{    0x0CDE,          0, 0},
	{    0x0CE0,     0x0CE3, 1},
	{    0x0CE6,     0x0CEF, 1},
	{    0x0CF1,     0x0CF2, 1},
	{    0x0D00,     0x0D03, 1},
	{    0x0D05,     0x0D0C, 1},
	{    0x0D0E,     0x0D10, 1},
	{    0x0D12,     0x0D3A, 1},
	{    0x0D3D,     0x0D44, 1},
	{    0x0D46,     0x0D48, 1},
	{    0x0D4A,     0x0D4C, 1},
	{    0x0D4E,          0, 0},
	{    0x0D54,     0x0D57, 1},
	{    0x0D5F,     0x0D63, 1},
	{    0x0D66,     0x0D6F, 1},
	{    0x0D7A,     0x0D7F, 1},
	{    0x0D82,     0x0D83, 1},
	{    0x0D85,     0x0D96, 1},
	{    0x0D9A,     0x0DB1, 1},
	{    0x0DB3,     0x0DBB, 1},
	{    0x0DBD,          0, 0},
	{    0x0DC0,     0x0DC6, 1},
	{    0x0DCF,     0x0DD4, 1},
	{    0x0DD6,          0, 0},
	{    0x0DD8,     0x0DDF, 1},
	{    0x0DE6,     0x0DEF, 1},
	{    0x0DF2,     0x0DF3, 1},
	{    0x0E01,     0x0E3A, 1},
	{    0x0E40,     0x0E46, 1},
	{    0x0E4D,          0, 0},
	{    0x0E50,     0x0E59, 1},
	{    0x0E81,     0x0E82, 1},
	{    0x0E84,          0, 0},
	{    0x0E87,     0x0E88, 1},
	{    0x0E8A,          0, 0},
	{    0x0E8D,          0, 0},
	{    0x0E94,     0x0E97, 1},
	{    0x0E99,     0x0E9F, 1},
	{    0x0EA1,     0x0EA3, 1},
	{    0x0EA5,          0, 0},
	{    0x0EA7,          0, 0},
	{    0x0EAA,     0x0EAB, 1},
	{    0x0EAD,     0x0EB9, 1},
	{    0x0EBB,     0x0EBD, 1},
	{    0x0EC0,     0x0EC4, 1},
	{    0x0EC6,          0, 0},
	{    0x0ECD,          0, 0},
	{    0x0ED0,     0x0ED9, 1},
	{    0x0EDC,     0x0EDF, 1},
	{    0x0F00,          0, 0},
	{    0x0F20,     0x0F29, 1},
	{    0x0F40,     0x0F47, 1},
	{    0x0F49,     0x0F6C, 1},
	{    0x0F71,     0x0F81, 1},
	{    0x0F88,     0x0F97, 1},
	{    0x0F99,     0x0FBC, 1},
	{    0x1000,     0x1036, 1},
	{    0x1038,          0, 0},
	{    0x103B,     0x1049, 1},
	{    0x1050,     0x1062, 1},
	{    0x1065,     0x1068, 1},
	{    0x106E,     0x1086, 1},
	{    0x108E,          0, 0},
	{    0x1090,     0x1099, 1},
	{    0x109C,     0x109D, 1},
	{    0x10A0,     0x10C5, 1},
	{    0x10C7,          0, 0},
	{    0x10CD,          0, 0},
	{    0x10D0,     0x10FA, 1},
	{    0x10FC,     0x1248, 1},
	{    0x124A,     0x124D, 1},
	{    0x1250,     0x1256, 1},
	{    0x1258,          0, 0},
	{    0x125A,     0x125D, 1},
	{    0x1260,     0x1288, 1},
	{    0x128A,     0x128D, 1},
	{    0x1290,     0x12B0, 1},
	{    0x12B2,     0x12B5, 1},
	{    0x12B8,     0x12BE, 1},
	{    0x12C0,          0, 0},
	{    0x12C2,     0x12C5, 1},
	{    0x12C8,     0x12D6, 1},
	{    0x12D8,     0x1310, 1},
	{    0x1312,     0x1315, 1},
	{    0x1318,     0x135A, 1},
	{    0x135F,          0, 0},
	{    0x1380,     0x138F, 1},
	{    0x13A0,     0x13F5, 1},
	{    0x13F8,     0x13FD, 1},
	{    0x1401,     0x166C, 1},
	{    0x166F,     0x167F, 1},
	{    0x1681,     0x169A, 1},
	{    0x16A0,     0x16EA, 1},
	{    0x16EE,     0x16F8, 1},
	{    0x1700,     0x170C, 1},
	{    0x170E,     0x1713, 1},
	{    0x1720,     0x1733, 1},
	{    0x1740,     0x1753, 1},
	{    0x1760,     0x176C, 1},
	{    0x176E,     0x1770, 1},
	{    0x1772,     0x1773, 1},
	{    0x1780,     0x17B3, 1},
	{    0x17B6,     0x17C8, 1},
	{    0x17D7,          0, 0},
	{    0x17DC,          0, 0},
	{    0x17E0,     0x17E9, 1},
	{    0x1810,     0x1819, 1},
	{    0x1820,     0x1877, 1},
	{    0x1880,     0x18AA, 1},
	{    0x18B0,     0x18F5, 1},
	{    0x1900,     0x191E, 1},
	{    0x1920,     0x192B, 1},
	{    0x1930,     0x1938, 1},
	{    0x1946,     0x196D, 1},
	{    0x1970,     0x1974, 1},
	{    0x1980,     0x19AB, 1},
	{    0x19B0,     0x19C9, 1},
	{    0x19D0,     0x19D9, 1},
	{    0x1A00,     0x1A1B, 1},
	{    0x1A20,     0x1A5E, 1},
	{    0x1A61,     0x1A74, 1},
	{    0x1A80,     0x1A89, 1},
	{    0x1A90,     0x1A99, 1},
	{    0x1AA7,          0, 0},
	{    0x1B00,     0x1B33, 1},
	{    0x1B35,     0x1B43, 1},
	{    0x1B45,     0x1B4B, 1},
	{    0x1B50,     0x1B59, 1},
	{    0x1B80,     0x1BA9, 1},
	{    0x1BAC,     0x1BE5, 1},
	{    0x1BE7,     0x1BF1, 1},
	{    0x1C00,     0x1C35, 1},
	{    0x1C40,     0x1C49, 1},
	{    0x1C4D,     0x1C7D, 1},
	{    0x1C80,     0x1C88, 1},
	{    0x1CE9,     0x1CEC, 1},
	{    0x1CEE,     0x1CF3, 1},
	{    0x1CF5,     0x1CF6, 1},
	{    0x1D00,     0x1DBF, 1},
	{    0x1DE7,     0x1DF4, 1},
	{    0x1E00,     0x1F15, 1},
	{    0x1F18,     0x1F1D, 1},
	{    0x1F20,     0x1F45, 1},
	{    0x1F48,     0x1F4D, 1},
	{    0x1F50,     0x1F57, 1},
	{    0x1F59,          0, 0},
	{    0x1F5B,          0, 0},
	{    0x1F5D,          0, 0},
	{    0x1F5F,     0x1F7D, 1},
	{    0x1F80,     0x1FB4, 1},
	{    0x1FB6,     0x1FBC, 1},
	{    0x1FBE,          0, 0},
	{    0x1FC2,     0x1FC4, 1},
	{    0x1FC6,     0x1FCC, 1},
	{    0x1FD0,     0x1FD3, 1},
	{    0x1FD6,     0x1FDB, 1},
	{    0x1FE0,     0x1FEC, 1},
	{    0x1FF2,     0x1FF4, 1},
	{    0x1FF6,     0x1FFC, 1},
	{    0x2071,          0, 0},
	{    0x207F,          0, 0},
	{    0x2090,     0x209C, 1},
	{    0x2102,          0, 0},
	{    0x2107,          0, 0},
	{    0x210A,     0x2113, 1},
	{    0x2115,          0, 0},
	{    0x2119,     0x211D, 1},
	{    0x2124,          0, 0},
	{    0x2126,          0, 0},
	{    0x2128,          0, 0},
	{    0x212A,     0x212D, 1},
	{    0x212F,     0x2139, 1},
	{    0x213C,     0x213F, 1},
	{    0x2145,     0x2149, 1},
	{    0x214E,          0, 0},
	{    0x2160,     0x2188, 1},
	{    0x24B6,     0x24E9, 1},
	{    0x2C00,     0x2C2E, 1},
	{    0x2C30,     0x2C5E, 1},
	{    0x2C60,     0x2CE4, 1},
	{    0x2CEB,     0x2CEE, 1},
	{    0x2CF2,     0x2CF3, 1},
	{    0x2D00,     0x2D25, 1},
	{    0x2D27,          0, 0},
	{    0x2D2D,          0, 0},
	{    0x2D30,     0x2D67, 1},
	{    0x2D6F,          0, 0},
	{    0x2D80,     0x2D96, 1},
	{    0x2DA0,     0x2DA6, 1},
	{    0x2DA8,     0x2DAE, 1},
	{    0x2DB0,     0x2DB6, 1},
	{    0x2DB8,     0x2DBE, 1},
	{    0x2DC0,     0x2DC6, 1},
	{    0x2DC8,     0x2DCE, 1},
	{    0x2DD0,     0x2DD6, 1},
	{    0x2DD8,     0x2DDE, 1},
	{    0x2DE0,     0x2DFF, 1},
	{    0x2E2F,          0, 0},
	{    0x3005,     0x3007, 1},
	{    0x3021,     0x3029, 1},
	{    0x3031,     0x3035, 1},
	{    0x3038,     0x303C, 1},
	{    0x3041,     0x3096, 1},
	{    0x309D,     0x309F, 1},
	{    0x30A1,     0x30FA, 1},
	{    0x30FC,     0x30FF, 1},
	{    0x3105,     0x312E, 1},
	{    0x3131,     0x318E, 1},
	{    0x31A0,     0x31BA, 1},
	{    0x31F0,     0x31FF, 1},
	{    0x3400,     0x4DB5, 1},
	{    0x4E00,     0x9FEA, 1},
	{    0xA000,     0xA48C, 1},
	{    0xA4D0,     0xA4FD, 1},
	{    0xA500,     0xA60C, 1},
	{    0xA610,     0xA62B, 1},
	{    0xA640,     0xA66E, 1},
	{    0xA674,     0xA67B, 1},
	{    0xA67F,     0xA6EF, 1},
	{    0xA717,     0xA71F, 1},
	{    0xA722,     0xA788, 1},
	{    0xA78B,     0xA7AE, 1},
	{    0xA7B0,     0xA7B7, 1},
	{    0xA7F7,     0xA801, 1},
	{    0xA803,     0xA805, 1},
	{    0xA807,     0xA80A, 1},
	{    0xA80C,     0xA827, 1},
	{    0xA840,     0xA873, 1},
	{    0xA880,     0xA8C3, 1},
	{    0xA8C5,          0, 0},
	{    0xA8D0,     0xA8D9, 1},
	{    0xA8F2,     0xA8F7, 1},
	{    0xA8FB,          0, 0},
	{    0xA8FD,          0, 0},
	{    0xA900,     0xA92A, 1},
	{    0xA930,     0xA952, 1},
	{    0xA960,     0xA97C, 1},
	{    0xA980,     0xA9B2, 1},
	{    0xA9B4,     0xA9BF, 1},
	{    0xA9CF,     0xA9D9, 1},
	{    0xA9E0,     0xA9E4, 1},
	{    0xA9E6,     0xA9FE, 1},
	{    0xAA00,     0xAA36, 1},
	{    0xAA40,     0xAA4D, 1},
	{    0xAA50,     0xAA59, 1},
	{    0xAA60,     0xAA76, 1},
	{    0xAA7A,          0, 0},
	{    0xAA7E,     0xAABE, 1},
	{    0xAAC0,          0, 0},
	{    0xAAC2,          0, 0},
	{    0xAADB,     0xAADD, 1},
	{    0xAAE0,     0xAAEF, 1},
	{    0xAAF2,     0xAAF5, 1},
	{    0xAB01,     0xAB06, 1},
	{    0xAB09,     0xAB0E, 1},
	{    0xAB11,     0xAB16, 1},
	{    0xAB20,     0xAB26, 1},
	{    0xAB28,     0xAB2E, 1},
	{    0xAB30,     0xAB5A, 1},
	{    0xAB5C,     0xAB65, 1},
	{    0xAB70,     0xABEA, 1},
	{    0xABF0,     0xABF9, 1},
	{    0xAC00,     0xD7A3, 1},
	{    0xD7B0,     0xD7C6, 1},
	{    0xD7CB,     0xD7FB, 1},
	{    0xF900,     0xFA6D, 1},
	{    0xFA70,     0xFAD9, 1},
	{    0xFB00,     0xFB06, 1},
	{    0xFB13,     0xFB17, 1},
	{    0xFB1D,     0xFB28, 1},
	{    0xFB2A,     0xFB36, 1},
	{    0xFB38,     0xFB3C, 1},
	{    0xFB3E,          0, 0},
	{    0xFB40,     0xFB41, 1},
	{    0xFB43,     0xFB44, 1},
	{    0xFB46,     0xFBB1, 1},
	{    0xFBD3,     0xFD3D, 1},
	{    0xFD50,     0xFD8F, 1},
	{    0xFD92,     0xFDC7, 1},
	{    0xFDF0,     0xFDFB, 1},
	{    0xFE70,     0xFE74, 1},
	{    0xFE76,     0xFEFC, 1},
	{    0xFF10,     0xFF19, 1},
	{    0xFF21,     0xFF3A, 1},
	{    0xFF41,     0xFF5A, 1},
	{    0xFF66,     0xFFBE, 1},
	{    0xFFC2,     0xFFC7, 1},
	{    0xFFCA,     0xFFCF, 1},
	{    0xFFD2,     0xFFD7, 1},
	{    0xFFDA,     0xFFDC, 1},
	{0x00010000, 0x0001000B, 1},
	{0x0001000D, 0x00010026, 1},
	{0x00010028, 0x0001003A, 1},
	{0x0001003C, 0x0001003D, 1},
	{0x0001003F, 0x0001004D, 1},
	{0x00010050, 0x0001005D, 1},
	{0x00010080, 0x000100FA, 1},
	{0x00010140, 0x00010174, 1},
	{0x00010280, 0x0001029C, 1},
	{0x000102A0, 0x000102D0, 1},
	{0x00010300, 0x0001031F, 1},
	{0x0001032D, 0x0001034A, 1},
	{0x00010350, 0x0001037A, 1},
	{0x00010380, 0x0001039D, 1},
	{0x000103A0, 0x000103C3, 1},
	{0x000103C8, 0x000103CF, 1},
	{0x000103D1, 0x000103D5, 1},
	{0x00010400, 0x0001049D, 1},
	{0x000104A0, 0x000104A9, 1},
	{0x000104B0, 0x000104D3, 1},
	{0x000104D8, 0x000104FB, 1},
	{0x00010500, 0x00010527, 1},
	{0x00010530, 0x00010563, 1},
	{0x00010600, 0x00010736, 1},
	{0x00010740, 0x00010755, 1},
	{0x00010760, 0x00010767, 1},
	{0x00010800, 0x00010805, 1},
	{0x00010808,          0, 0},
	{0x0001080A, 0x00010835, 1},
	{0x00010837, 0x00010838, 1},
	{0x0001083C,          0, 0},
	{0x0001083F, 0x00010855, 1},
	{0x00010860, 0x00010876, 1},
	{0x00010880, 0x0001089E, 1},
	{0x000108E0, 0x000108F2, 1},
	{0x000108F4, 0x000108F5, 1},
	{0x00010900, 0x00010915, 1},
	{0x00010920, 0x00010939, 1},
	{0x00010980, 0x000109B7, 1},
	{0x000109BE, 0x000109BF, 1},
	{0x00010A00, 0x00010A03, 1},
	{0x00010A05, 0x00010A06, 1},
	{0x00010A0C, 0x00010A13, 1},
	{0x00010A15, 0x00010A17, 1},
	{0x00010A19, 0x00010A33, 1},
	{0x00010A60, 0x00010A7C, 1},
	{0x00010A80, 0x00010A9C, 1},
	{0x00010AC0, 0x00010AC7, 1},
	{0x00010AC9, 0x00010AE4, 1},
	{0x00010B00, 0x00010B35, 1},
	{0x00010B40, 0x00010B55, 1},
	{0x00010B60, 0x00010B72, 1},
	{0x00010B80, 0x00010B91, 1},
	{0x00010C00, 0x00010C48, 1},
	{0x00010C80, 0x00010CB2, 1},
	{0x00010CC0, 0x00010CF2, 1},
	{0x00011000, 0x00011045, 1},
	{0x00011066, 0x0001106F, 1},
	{0x00011082, 0x000110B8, 1},
	{0x000110D0, 0x000110E8, 1},
	{0x000110F0, 0x000110F9, 1},
	{0x00011100, 0x00011132, 1},
	{0x00011136, 0x0001113F, 1},
	{0x00011150, 0x00011172, 1},
	{0x00011176,          0, 0},
	{0x00011180, 0x000111BF, 1},
	{0x000111C1, 0x000111C4, 1},
	{0x000111D0, 0x000111DA, 1},
	{0x000111DC,          0, 0},
	{0x00011200, 0x00011211, 1},
	{0x00011213, 0x00011234, 1},
	{0x00011237,          0, 0},
	{0x0001123E,          0, 0},
	{0x00011280, 0x00011286, 1},
	{0x00011288,          0, 0},
	{0x0001128A, 0x0001128D, 1},
	{0x0001128F, 0x0001129D, 1},
	{0x0001129F, 0x000112A8, 1},
	{0x000112B0, 0x000112E8, 1},
	{0x000112F0, 0x000112F9, 1},
	{0x00011300, 0x00011303, 1},
	{0x00011305, 0x0001130C, 1},
	{0x0001130F, 0x00011310, 1},
	{0x00011313, 0x00011328, 1},
	{0x0001132A, 0x00011330, 1},
	{0x00011332, 0x00011333, 1},
	{0x00011335, 0x00011339, 1},
	{0x0001133D, 0x00011344, 1},
	{0x00011347, 0x00011348, 1},
	{0x0001134B, 0x0001134C, 1},
	{0x00011350,          0, 0},
	{0x00011357,          0, 0},
	{0x0001135D, 0x00011363, 1},
	{0x00011400, 0x00011441, 1},
	{0x00011443, 0x00011445, 1},
	{0x00011447, 0x0001144A, 1},
	{0x00011450, 0x00011459, 1},
	{0x00011480, 0x000114C1, 1},
	{0x000114C4, 0x000114C5, 1},
	{0x000114C7,          0, 0},
	{0x000114D0, 0x000114D9, 1},
	{0x00011580, 0x000115B5, 1},
	{0x000115B8, 0x000115BE, 1},
	{0x000115D8, 0x000115DD, 1},
	{0x00011600, 0x0001163E, 1},
	{0x00011640,          0, 0},
	{0x00011644,          0, 0},
	{0x00011650, 0x00011659, 1},
	{0x00011680, 0x000116B5, 1},
	{0x000116C0, 0x000116C9, 1},
	{0x00011700, 0x00011719, 1},
	{0x0001171D, 0x0001172A, 1},
	{0x00011730, 0x00011739, 1},
	{0x000118A0, 0x000118E9, 1},
	{0x000118FF,          0, 0},
	{0x00011A00, 0x00011A32, 1},
	{0x00011A35, 0x00011A3E, 1},
	{0x00011A50, 0x00011A83, 1},
	{0x00011A86, 0x00011A97, 1},
	{0x00011AC0, 0x00011AF8, 1},
	{0x00011C00, 0x00011C08, 1},
	{0x00011C0A, 0x00011C36, 1},
	{0x00011C38, 0x00011C3E, 1},
	{0x00011C40,          0, 0},
	{0x00011C50, 0x00011C59, 1},
	{0x00011C72, 0x00011C8F, 1},
	{0x00011C92, 0x00011CA7, 1},
	{0x00011CA9, 0x00011CB6, 1},
	{0x00011D00, 0x00011D06, 1},
	{0x00011D08, 0x00011D09, 1},
	{0x00011D0B, 0x00011D36, 1},
	{0x00011D3A,          0, 0},
	{0x00011D3C, 0x00011D3D, 1},
	{0x00011D3F, 0x00011D41, 1},
	{0x00011D43,          0, 0},
	{0x00011D46, 0x00011D47, 1},
	{0x00011D50, 0x00011D59, 1},
	{0x00012000, 0x00012399, 1},
	{0x00012400, 0x0001246E, 1},
	{0x00012480, 0x00012543, 1},
	{0x00013000, 0x0001342E, 1},
	{0x00014400, 0x00014646, 1},
	{0x00016800, 0x00016A38, 1},
	{0x00016A40, 0x00016A5E, 1},
	{0x00016A60, 0x00016A69, 1},
	{0x00016AD0, 0x00016AED, 1},
	{0x00016B00, 0x00016B36, 1},
	{0x00016B40, 0x00016B43, 1},
	{0x00016B50, 0x00016B59, 1},
	{0x00016B63, 0x00016B77, 1},
	{0x00016B7D, 0x00016B8F, 1},
	{0x00016F00, 0x00016F44, 1},
	{0x00016F50, 0x00016F7E, 1},
	{0x00016F93, 0x00016F9F, 1},
	{0x00016FE0, 0x00016FE1, 1},
	{0x00017000, 0x000187EC, 1},
	{0x00018800, 0x00018AF2, 1},
	{0x0001B000, 0x0001B11E, 1},
	{0x0001B170, 0x0001B2FB, 1},
	{0x0001BC00, 0x0001BC6A, 1},
	{0x0001BC70, 0x0001BC7C, 1},
	{0x0001BC80, 0x0001BC88, 1},
	{0x0001BC90, 0x0001BC99, 1},
	{0x0001BC9E,          0, 0},
	{0x0001D400, 0x0001D454, 1},
	{0x0001D456, 0x0001D49C, 1},
	{0x0001D49E, 0x0001D49F, 1},
	{0x0001D4A2,          0, 0},
	{0x0001D4A5, 0x0001D4A6, 1},
	{0x0001D4A9, 0x0001D4AC, 1},
	{0x0001D4AE, 0x0001D4B9, 1},
	{0x0001D4BB,          0, 0},
	{0x0001D4BD, 0x0001D4C3, 1},
	{0x0001D4C5, 0x0001D505, 1},
	{0x0001D507, 0x0001D50A, 1},
	{0x0001D50D, 0x0001D514, 1},
	{0x0001D516, 0x0001D51C, 1},
	{0x0001D51E, 0x0001D539, 1},
	{0x0001D53B, 0x0001D53E, 1},
	{0x0001D540, 0x0001D544, 1},
	{0x0001D546,          0, 0},
	{0x0001D54A, 0x0001D550, 1},
	{0x0001D552, 0x0001D6A5, 1},
	{0x0001D6A8, 0x0001D6C0, 1},
	{0x0001D6C2, 0x0001D6DA, 1},
	{0x0001D6DC, 0x0001D6FA, 1},
	{0x0001D6FC, 0x0001D714, 1},
	{0x0001D716, 0x0001D734, 1},
	{0x0001D736, 0x0001D74E, 1},
	{0x0001D750, 0x0001D76E, 1},
	{0x0001D770, 0x0001D788, 1},
	{0x0001D78A, 0x0001D7A8, 1},
	{0x0001D7AA, 0x0001D7C2, 1},
	{0x0001D7C4, 0x0001D7CB, 1},
	{0x0001D7CE, 0x0001D7FF, 1},
	{0x0001E000, 0x0001E006, 1},
	{0x0001E008, 0x0001E018, 1},
	{0x0001E01B, 0x0001E021, 1},
	{0x0001E023, 0x0001E024, 1},
	{0x0001E026, 0x0001E02A, 1},
	{0x0001E800, 0x0001E8C4, 1},
	{0x0001E900, 0x0001E943, 1},
	{0x0001E947,          0, 0},
	{0x0001E950, 0x0001E959, 1},
	{0x0001EE00, 0x0001EE03, 1},
	{0x0001EE05, 0x0001EE1F, 1},
	{0x0001EE21, 0x0001EE22, 1},
	{0x0001EE24,          0, 0},
	{0x0001EE27,          0, 0},
	{0x0001EE29, 0x0001EE32, 1},
	{0x0001EE34, 0x0001EE37, 1},
	{0x0001EE39,          0, 0},
	{0x0001EE3B,          0, 0},
	{0x0001EE42,          0, 0},
	{0x0001EE47,          0, 0},
	{0x0001EE49,          0, 0},
	{0x0001EE4B,          0, 0},
	{0x0001EE4D, 0x0001EE4F, 1},
	{0x0001EE51, 0x0001EE52, 1},
	{0x0001EE54,          0, 0},
	{0x0001EE57,          0, 0},
	{0x0001EE59,          0, 0},
	{0x0001EE5B,          0, 0},
	{0x0001EE5D,          0, 0},
	{0x0001EE5F,          0, 0},
	{0x0001EE61, 0x0001EE62, 1},
	{0x0001EE64,          0, 0},
	{0x0001EE67, 0x0001EE6A, 1},
	{0x0001EE6C, 0x0001EE72, 1},
	{0x0001EE74, 0x0001EE77, 1},
	{0x0001EE79, 0x0001EE7C, 1},
	{0x0001EE7E,          0, 0},
	{0x0001EE80, 0x0001EE89, 1},
	{0x0001EE8B, 0x0001EE9B, 1},
	{0x0001EEA1, 0x0001EEA3, 1},
	{0x0001EEA5, 0x0001EEA9, 1},
	{0x0001EEAB, 0x0001EEBB, 1},
	{0x0001F130, 0x0001F149, 1},
	{0x0001F150, 0x0001F169, 1},
	{0x0001F170, 0x0001F189, 1},
	{0x00020000, 0x0002A6D6, 1},
	{0x0002A700, 0x0002B734, 1},
	{0x0002B740, 0x0002B81D, 1},
	{0x0002B820, 0x0002CEA1, 1},
	{0x0002CEB0, 0x0002EBE0, 1},
	{0x0002F800, 0x0002FA1D, 1},

/* END::UTF8TABLE */
};

static inline size_t utf8towc(unsigned *wc, const unsigned char *uc, size_t len)
{
	unsigned char ub = utf8_mblen[*uc];

	if (!ub || ub > len || ub > 4) {
		return 0;
	}

	*wc = *uc & utf8_mask[ub];

	switch (ub) {
	case 4:
		if ((uc[1] & 0xc0) != 0x80) {
			return 0;
		}
		*wc <<= 6;
		*wc += *++uc & 0x3f;
		/* no break */
	case 3:
		if ((uc[1] & 0xc0) != 0x80) {
			return 0;
		}
		*wc <<= 6;
		*wc += *++uc & 0x3f;
		/* no break */
	case 2:
		if ((uc[1] & 0xc0) != 0x80) {
			return 0;
		}
		*wc <<= 6;
		*wc += *++uc & 0x3f;
		/* no break */
	case 1:
		break;

	default:
		return 0;
	}

	return ub;
}

static inline zend_bool isualpha(unsigned ch)
{
	unsigned j;
	const utf8_range_t *u = &utf8_ranges[0], *e = &utf8_ranges[sizeof(utf8_ranges)/sizeof(utf8_range_t)-1];

	do {
		if (u->start == ch) {
			return 1;
		} else if (u->start <= ch && u->end >= ch) {
			if (u->step == 1) {
				return 1;
			}
			for (j = u->start; j <= u->end; j+= u->step) {
				if (ch == j) {
					return 1;
				}
			}
			return 0;
		}
	} while (++u != e);

	return 0;
}

static inline zend_bool isualnum(unsigned ch)
{
	/* digits */
	if (ch >= 0x30 && ch <= 0x39) {
		return 1;
	}
	return isualpha(ch);
}

static inline size_t wctoutf16(unsigned short u16[2], unsigned wc)
{
	if (wc > 0x10ffff || (wc >= 0xd800 && wc <= 0xdfff)) {
		return 0;
	}

	if (wc <= 0xffff) {
		u16[0] = (unsigned short) wc;
		return 1;
	}

	wc -= 0x10000;
	u16[0] = (unsigned short) ((wc >> 10) + 0xd800);
	u16[1] = (unsigned short) ((wc & 0x3ff) + 0xdc00);
	return 2;
}

static inline size_t utf16towc(unsigned *wc, unsigned short *u16_str, size_t u16_len)
{
	if (u16_len < 1) {
		return 0;
	}
	if (u16_str[0] - 0xd800 >= 0x800) {
		*wc = u16_str[0];
		return 1;
	}
	if (u16_len < 2 || (u16_str[0] & 0xfffffc00) != 0xd800 || (u16_str[1] & 0xfffffc00) != 0xdc00) {
		return 0;
	}
	*wc = (u16_str[0] << 10) + u16_str[1] - 0x35fdc00;
	return 2;
}

#endif	/* PHP_HTTP_UTF8_H */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
